/*****************************************************************
 * Copyright (c) ParaSoft Corporation 1991.  All rights reserved.*
 * THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF ParaSoft       *
 * The copyright notice above does not evidence any              *
 * actual or intended publication of such source code.           *
 *****************************************************************/

#include "extern.h"

Node *se_AttrSpec_list = NN;
int se_AttrSpec_opt = 0;
int se_AttrSpec_param = 0;
int se_AttrSpec_external = 0;
int se_AttrSpec_intrinsic = 0;
int se_AttrSpec_save = 0;

int se_AttrSpec_public = 0;
int se_AttrSpec_private = 0;
int se_AttrSpec_allocatable = 0;
int se_AttrSpec_data = 0;
int se_AttrSpec_intent = 0;
int se_AttrSpec_pointer = 0;
int se_AttrSpec_target = 0;

int se_is_statement_function_def = 0;
int se_in_ComplexLiteralConstant = 0;

Etype *se_EtypeSpec = EN;
int se_DimensionStmt_in = 0;
int se_CommonStmt_in = 0;
int se_CommonStmt_blank = 0;
Typ *se_Where_typ;

int se_in_interface = 0;

clear_se ()  /* called at the end of every statement during parsing  */
{
	se_AttrSpec_list = NN;
	se_AttrSpec_opt = 0;
	se_AttrSpec_param = 0;
	se_AttrSpec_external = 0;
	se_AttrSpec_intrinsic = 0;
	se_AttrSpec_save = 0;

	se_AttrSpec_public = 0;
	se_AttrSpec_private = 0;
	se_AttrSpec_allocatable = 0;
	se_AttrSpec_data = 0;
	se_AttrSpec_intent = 0;
	se_AttrSpec_pointer = 0;
	se_AttrSpec_target = 0;

	se_EtypeSpec = EN;
	se_DimensionStmt_in = 0;
	se_CommonStmt_in = 0;
	se_CommonStmt_blank = 0;

	se_in_ForallStmt = 0;
	se_is_statement_function_def = 0;
}

/*
Parsing routines used directly in yacc spec, this file should be called parse.c at
some point.
*/

Node *parse_TypeSpec (root, name, type, def)
Node *root;
char *name;
int type, def;
{
	Node *ks = root->TypeSpec.OptKindSelector;

	if (ks != NN) {
		int kind, nbytes;

		if (ks->KindSelector.IntegerConstant == NN) {
			f90ni_KIND;
			nbytes = 4;
		}
		else {
			nbytes = atoi (ks->KindSelector.IntegerConstant->com.str);
		}

		sprintf (serr, "length specification on %s type", name);
		stand (serr);

		if ((kind = machine_type_kind (type, nbytes)) < 0) {
/* error recovery because KIND is not implemented */
#ifndef CUBIX
			sprintf (serr, "type %s*%d not available"
			       , name, nbytes);
			fatal (serr);
#endif
			se_EtypeSpec = machine_etype (type, def);
			return root;
		}
		else {
			se_EtypeSpec = machine_etype (type, kind);
		}
	}
	else {
		se_EtypeSpec = machine_etype (type, def);

		if (mac_etype_size (se_EtypeSpec) < 0)
			SYSERR;
	}

	return make_KeyName (mac_etype_name (se_EtypeSpec));
}

yacc_AttrSpec (node)
Node *node;
{
	char *orig = node->AttrSpec.Name->com.str;
	char *name = U_str_copy_upper (orig);
	int public, private, allocatable, dimension, intent, pointer, target;
	int has_par = (node->AttrSpec.Lp != NN) ? 1 : 0;

	public = (strcmp (name, "PUBLIC") == 0) ? 1 : 0;
	private = (strcmp (name, "PRIVATE") == 0) ? 1 : 0;
	allocatable = (strcmp (name, "ALLOCATABLE") == 0) ? 1 : 0;
	dimension = (strcmp (name, "DIMENSION") == 0 || strcmp (name, "ARRAY") == 0)
	? 1 : 0;
	intent = (strcmp (name, "INTENT") == 0) ? 1 : 0;
	pointer = (strcmp (name, "POINTER") == 0) ? 1 : 0;
	target = (strcmp (name, "TARGET") == 0) ? 1 : 0;

	if (strcmp (name, "PARAMETER")
	&&  !public
	&&  !private
	&&  !allocatable
	&&  !dimension
	&&  strcmp (name, "EXTERNAL")
	&&  !intent
	&&  strcmp (name, "INTRINSIC")
	&&  strcmp (name, "OPTIONAL")
	&&  !pointer
	&&  strcmp (name, "SAVE")
	&&  !target
	&&  strcmp (name, "DATA")
	) {
		sprintf (serr, "wrong attribute-specification: %s", orig);
		fatal (serr);
		return;
	}

	if ((!dimension) && (!intent) && has_par) {
		sprintf (serr, "%s attribute cannot have '(...)'", name);
		fatal (serr);
	}

	if (public || private)
		f90ni_UDDTAO;
	else if (allocatable || pointer || target)
		f90ni_POINTERS;
	else if (dimension && !has_par)
		fatal ("missing dimension-list in DIMENSION attribute");
	else if (intent) {
		Node *sublist, *Nname = NN;
		char *intent_name;

		if (!has_par)
			fatal ("missing intent-specification in INTENT attribute");
		else {
			sublist = node->AttrSpec.SubscriptList;

			if (sublist == NN || sublist->lst.next != NN
			||  sublist->lst.value->Subscript.Expr1 != NN
			||  sublist->lst.value->Subscript.Expr2 != NN
			||  sublist->lst.value->Subscript.Colon != NN
			||  sublist->lst.value->Subscript.Colon1 != NN
			||  sublist->lst.value->Subscript.Expr->com.type != VARIABLE)
				fatal ("wrong intent-specification");
			else if
			(intent_type ((Nname = sublist->lst.value->Subscript.Expr->Variable.Name)->com.str) < 0)
				fatal ("INTENT specification must be IN, OUT or INOUT");
			else  /* to avoid parse_name in check_pass */
				node->AttrSpec.SubscriptList = Nname;
		}
	}
}

static int yacc_CharSelector_is_kind (name)
char *name;
{
	return 0;
}

Node *yacc_CharSelector_2 (node)
Node *node;
{
	Node *new = NN;

	if (!yacc_CharSelector_is_kind (node->CharSelector.Name)) {
		new = make (CHAR_SELECTOR);
		new->CharSelector.Times = &Times;
		new->CharSelector.CharLength = node->CharSelector.Expr;;
		return new;
	}

	if (!yacc_CharSelector_is_kind (node->CharSelector.Name1)) {
		if (new != NN) {
			fatal ("too many LENs in length-selector");
			return new;
		}

		new = make (CHAR_SELECTOR);
		new->CharSelector.Times = &Times;
		new->CharSelector.CharLength = node->CharSelector.Expr1;;
		return new;
	}

	return node;
}

Node *yacc_CharSelector_1 (node)
Node *node;
{
	Node *new = NN;

	if (!yacc_CharSelector_is_kind (node->CharSelector.Name)) {
		new = make (CHAR_SELECTOR);
		new->CharSelector.Times = &Times;
		new->CharSelector.CharLength = node->CharSelector.Expr;;
		return new;
	}

	return node;
}
